package com.starcharge.wios.controller;

import com.alibaba.fastjson.JSONObject;
import com.ihidea.core.util.HttpClientUtils;
import com.starcharge.wios.dao.entity.Delivery;
import com.starcharge.wios.dao.mappers.DeliveryMapper;
import com.starcharge.wios.dto.DeliveryUpdateDTO;
import io.netty.util.internal.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.bo.OrderInstallBo;
import com.starcharge.wios.service.OrderInstallService;
import com.starcharge.wios.vo.OrderInstallVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import java.util.HashMap;
import java.util.Map;

/**
 * 安装订单管理接口
 * 
 * @author xgl
 * @version [版本号, 2020年9月25日]
 */
@RestController
@RequestMapping("api")
@Api(tags = "安装订单管理接口")
public class OrderInstallController {
    @Autowired
    private OrderInstallService orderInstallService;
    @Autowired
    private DeliveryMapper deliveryMapper;
    @Value("${warehouseUrl.reduceUrl}")
    private String reduceUrl;
    
    /**
     * 添加安装订单
     *
     * @param orderInstall
     * @return
     */
    @PostMapping(value = "orderInstall/add")
    @ApiOperation(value = "添加安装订单信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单id", dataType = "String", paramType = "query", required = true)})
    public BaseResponse<Object> add(OrderInstallBo orderInstall) {
        Assert.notNull(orderInstall.getOrderId(), "orderId不能为空");
        if (!StringUtils.isEmpty(orderInstall.getPredictOverItem())) {
            orderInstall.setPredictOverItem(StringUtilsEx.unescapeXss(orderInstall.getPredictOverItem()));
        }
        if (!StringUtils.isEmpty(orderInstall.getDebugCheck())) {
            orderInstall.setDebugCheck(StringUtilsEx.unescapeXss(orderInstall.getDebugCheck()));
        }
        orderInstallService.add(orderInstall);
        return new BaseResponse<>();
    }
    
    /**
     * 修改安装订单
     *
     * @param orderInstall
     * @return
     */
    @PostMapping(value = "orderInstall/update")
    @ApiOperation(value = "修改安装订单信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "安装订单id", dataType = "int", paramType = "query", required = true)})
    public BaseResponse<Object> update(OrderInstallBo orderInstall) {
        if (orderInstall.getId() == null) {
            throw new ServiceException("缺少参数");
        }
        if (!StringUtils.isEmpty(orderInstall.getPredictOverItem())) {
            orderInstall.setPredictOverItem(StringUtilsEx.unescapeXss(orderInstall.getPredictOverItem()));
        }
        if (!StringUtils.isEmpty(orderInstall.getDebugCheck())) {
            orderInstall.setDebugCheck(StringUtilsEx.unescapeXss(orderInstall.getDebugCheck()));
        }
        // 已支付订单不能修改增项信息
        if (orderInstall.getParkingType() != null && orderInstall.getParkingType().intValue() == 1) {
            orderInstall.setPredictOverItem(null);
        }
        orderInstallService.update(orderInstall.getId(), orderInstall);
        return new BaseResponse<>();
    }
    
    /**
     * 查询安装订单详情
     *
     * @param orderId
     * @return
     */
    @GetMapping(value = "orderInstall/detail")
    @ApiOperation(value = "查询安装订单详情")
    @ApiImplicitParam(paramType = "query", name = "orderId", value = "订单id", required = true, dataType = "String")
    public BaseResponse<OrderInstallVo> get(String orderId) {
        Assert.notNull(orderId, "orderId不能为空");
        OrderInstallVo orderInstall = orderInstallService.getByOrderId(orderId);
        return new BaseResponse<OrderInstallVo>(orderInstall);
    }
    /**
     * 校验墙盒SN号
     *
     * @param materialSn
     * @return
     */
    @PostMapping(value = "orderInstall/instaallReport")
    @ApiOperation(value = "校验墙盒SN号")
    public BaseResponse<Object> instaallReport(String materialSn) {
        Delivery delivery=this.deliveryMapper.selectByMaterialSn(materialSn);
        if (StringUtils.isEmpty(delivery)){
            throw new ServiceException(200,"没有当前SN号");
        }
        if (delivery.getInstallStatus()!=0){
            throw new ServiceException(200,"当前状态不是未安装状态");
        }
        //到货仓库的库存减少
        Delivery before=this.deliveryMapper.selectByPrimaryKey(delivery.getId());
        Map<String, String> params = new HashMap<String, String>();
        params.put("warehouseId",before.getWarehouseId());
        params.put("materialsCode",before.getMaterialCode());
        params.put("quantity","1");
        String jsonReduce = HttpClientUtils.post(reduceUrl,params,"UTF-8", "UTF-8");
        JSONObject jsonReduceObject=JSONObject.parseObject(jsonReduce);
        String codeReduce=jsonReduceObject.getString("code");
        if (StringUtil.isNullOrEmpty(codeReduce)){
            throw new ServiceException("到货仓库库存扣减失败");
        }
        DeliveryUpdateDTO deliveryUpdateDTO=new DeliveryUpdateDTO();
        deliveryUpdateDTO.setId(delivery.getId());
        deliveryUpdateDTO.setInstallStatus(1);
        this.deliveryMapper.updateByPrimaryKeySelective(deliveryUpdateDTO);
        return new BaseResponse<>();
    }
}
