package com.starcharge.wios.controller;

import com.alibaba.fastjson.JSONArray;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.TauxiliarymaterialsConvert;
import com.starcharge.wios.dao.entity.Delivery;
import com.starcharge.wios.dao.entity.Tauxiliarymaterials;
import com.starcharge.wios.dao.entity.TauxiliarymaterialsCriteria;
import com.starcharge.wios.dao.mappers.DeliveryMapper;
import com.starcharge.wios.dao.mappers.TauxiliarymaterialsMapper;
import com.starcharge.wios.dto.TauxiliarymaterialsCheckDto;
import com.starcharge.wios.enums2.LogisticCompany;
import com.starcharge.wios.enums2.TauxiliarymaterialStatusEnum;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.vo.TauxiliarymaterialsExcelVo;
import com.starcharge.wios.vo.TauxiliarymaterialsVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.var;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/9/25 14:50
 */
@RestController
@RequestMapping("api/tauxiliarymaterials")
@Api(tags = "辅材管理接口")
public class TauxiliarymaterialsController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private TauxiliarymaterialsMapper tauxiliarymaterialsMapper;
    @Autowired
    private DeliveryMapper deliveryMapper;

    public void UpdateColumns(Tauxiliarymaterials tauxiliarymaterials){
        SessionInfo sessionInfo = tokenService.getUser();
        tauxiliarymaterials.setUpdatedBy(sessionInfo.getUserId());
        tauxiliarymaterials.setUpdatedTime(LocalDateTime.now());
    }


    @PostMapping("/add")
    @ApiOperation("新增辅材申请")
    @ParamsValidate
    public BaseResponse<Tauxiliarymaterials> AddTauxiliarymaterials( @RequestBody @Validated(CreateEntityGroup.class) TauxiliarymaterialsVo tauxiliarymaterialsVo, BindingResult bindingResult){
        SessionInfo sessionInfo = tokenService.getUser();
        var tauxiliarymaterials=TauxiliarymaterialsConvert.INSTANCE.convertListVo(tauxiliarymaterialsVo);
        var now=LocalDateTime.now();
        tauxiliarymaterials.setApplyStatus(TauxiliarymaterialStatusEnum.已申请.name());
        tauxiliarymaterials.setLogisticStatus(TauxiliarymaterialStatusEnum.未发货.name());
        tauxiliarymaterials.setReceivingStatus(TauxiliarymaterialStatusEnum.未收货.name());
        tauxiliarymaterials.setApplyTime(now);
        tauxiliarymaterials.setCreatedBy(sessionInfo.getUserId());
        tauxiliarymaterials.setCreatedTime(now);
        tauxiliarymaterials.setUpdatedBy(sessionInfo.getUserId());
        tauxiliarymaterials.setUpdatedTime(now);
        tauxiliarymaterials.setAreaName(tauxiliarymaterialsVo.getAreaName());
        tauxiliarymaterials.setWarehouseName(tauxiliarymaterialsVo.getWarehouseName());
        //暂时代替申请单号
        tauxiliarymaterials.setApplyOrder("FC"+NumberUtil.getRandomOrderId());
        tauxiliarymaterialsMapper.insertSelective(tauxiliarymaterials);
        return new BaseResponse<>(tauxiliarymaterials);
    }
    @GetMapping("/detail")
    @ApiOperation(value = "查询辅材申请详情")
    public BaseResponse<Tauxiliarymaterials> detail(@RequestParam int id){
        Tauxiliarymaterials tauxiliarymaterials=this.tauxiliarymaterialsMapper.selectByPrimaryKey(id);
        if (tauxiliarymaterials.getLogisticOrder()!=null){
            List<String> listLogistic=new ArrayList<>();
           String [] list=tauxiliarymaterials.getLogisticOrder().split(",");
           for (String list1:list){
               if (!"<string>".equals(list1) && !"".equals(list1) && list1!=null) {
                   //Delivery delivery=deliveryMapper.selectByLogisticOrder(list1);
                   //if (delivery!=null){
                   listLogistic.add(list1 + "-" + LogisticCompany.德邦物流);
                   //}
               }
           }
            tauxiliarymaterials.setLogisticIdAndCompany(listLogistic);
        }
        return new BaseResponse(tauxiliarymaterials);
    }

    @PostMapping("/query")
    @ApiOperation(value = "查询辅材申请列表",notes = "属性不为空即为查询条件")
    public BaseResponse<List<Tauxiliarymaterials>> queryTauxiliarymaterialsList(Tauxiliarymaterials tauxiliarymaterials){
        List<Tauxiliarymaterials> list=this.tauxiliarymaterialsMapper.selectTAuxiliaryMaterialsList(tauxiliarymaterials);
        LocalDateTime currentTime = LocalDateTime.now();
        for (Tauxiliarymaterials apply : list) {
            LocalDateTime createdDate = apply.getCreatedTime();
            ZoneId zoneId=ZoneId.systemDefault();
            Date date=Date.from(createdDate.atZone(zoneId).toInstant());
            LocalDateTime createdTime = LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());
            LocalDateTime endTime = createdTime.plusHours(24);
            long remainingHours = currentTime.until(endTime, ChronoUnit.HOURS);
            remainingHours = Math.max(remainingHours, 0);  // 将剩余小时数限制在0-24之间
            if (remainingHours == 0) {
                apply.setIfTimeout(1L);
            } else {
                apply.setIfTimeout(0L);
            }
            apply.setTimeRemaining(Long.toString(remainingHours));
        }
        return new BaseResponse(list);
    }
    @PostMapping("/delete")
    @ApiOperation(value = "删除辅材申请")
    public BaseResponse<Boolean> deleteTauxiliarymaterials(@RequestParam int id){
        return new BaseResponse(this.tauxiliarymaterialsMapper.deleteByPrimaryKey(id)>0);
    }
    @PostMapping("/updateinfo")
    @ApiOperation(value = "更新辅材申请信息")
    public BaseResponse<Boolean> updateTauxiliarymaterialsInfo(@RequestBody TauxiliarymaterialsVo tauxiliarymaterials){
        if(tauxiliarymaterials.getId()==null||tauxiliarymaterials.getId()<=0){
            throw new ServiceException("400","id不能为空");
        }
        var updateDto=new Tauxiliarymaterials();
        BeanUtils.copyProperties(tauxiliarymaterials,updateDto);
        UpdateColumns(updateDto);
        return new BaseResponse(this.tauxiliarymaterialsMapper.updateByPrimaryKeySelective(updateDto)>0);
    }
    @GetMapping("/updatestatus")
    @ApiOperation(value = "更新辅材申请状态")
    public BaseResponse<Boolean> updateStatus(@RequestParam int id,
                                @RequestParam(required = false) @ApiParam("发货状态：未发货,已发货；可为空") String logisticStatus,
                                @RequestParam(required = false) @ApiParam("收货状态：未收货, 已收货；可为空")String receivingStatus,
                                @RequestParam(required = false) @ApiParam("申请状态：已申请,已驳回,已通过,已关闭；可为空")String applyStatus){
        if(StringUtils.isEmpty(logisticStatus)&&StringUtils.isEmpty(receivingStatus)&&StringUtils.isEmpty(applyStatus)){
            throw new ServiceException("400","至少更新一种状态");
        }
        var ta=new Tauxiliarymaterials();
        ta.setId(id);
        ta.setApplyStatus(TauxiliarymaterialStatusEnum.getEnumNameByName(0,applyStatus));
        ta.setLogisticStatus(TauxiliarymaterialStatusEnum.getEnumNameByName(1,logisticStatus));
        ta.setReceivingStatus(TauxiliarymaterialStatusEnum.getEnumNameByName(2,receivingStatus));
        UpdateColumns(ta);

        return new BaseResponse(this.tauxiliarymaterialsMapper.updateByPrimaryKeySelective(ta)>0);
    }
    @PostMapping("/check")
    @ApiOperation(value = "批量审批")
    @Transactional
    public BaseResponse<Boolean> checkApply(@RequestBody TauxiliarymaterialsCheckDto tauxiliarymaterialsCheckDto){
        SessionInfo sessionInfo = tokenService.getUser();
        tauxiliarymaterialsCheckDto.getTauxiliarymaterialsCheckVoList().stream()
                .map(dto->{
                    var ta=new Tauxiliarymaterials();
                    ta.setId(dto.getId());
                    ta.setApplyOrder(dto.getApplyOrder());
                    ta.setApprovedQuantity(dto.getApprovedQuantity());
                    ta.setApplyStatus(tauxiliarymaterialsCheckDto.getApplyStatus());
                    ta.setCheckSuggestion(tauxiliarymaterialsCheckDto.getCheckSuggestion());
                    ta.setCheckUser(sessionInfo.getUserId());
                    ta.setCheckUserName(sessionInfo.getUserName());
                    ta.setCheckTime(LocalDateTime.now());
                    UpdateColumns(ta);

                    Tauxiliarymaterials exist=null;
                    if(dto.getId()!=null&&dto.getId()>0){
                        exist=this.tauxiliarymaterialsMapper.selectByPrimaryKey(dto.getId());
                    }else if(StringUtils.hasLength(dto.getApplyOrder())){
                        exist=this.tauxiliarymaterialsMapper.selectByApplyOrder(dto.getApplyOrder());
                    }
                    if(exist==null){
                        throw new ServiceException(400,"找不到辅材申请单");
                    }else if(TauxiliarymaterialStatusEnum.getEnumByName(0,exist.getApplyStatus()).getValue()!=1){
                        throw new ServiceException(400,"辅材申请单:"+exist.getApplyOrder()+" 状态必须是已申请");
                    }else{
                        ta.setId(exist.getId());
                    }
                    return ta;
                })
                .filter(ta->{
                    if(ta.getId()==null||ta.getId()<=0){
                        return false;
                    }else{
                        return true;
                    }
                })
                .forEach(ta->{
                    this.tauxiliarymaterialsMapper.updateByPrimaryKeySelective(ta);
                });
        return new BaseResponse(true);
    }
    @GetMapping("/bindLogisticOrder")
    @ApiOperation(value = "绑定物流单号")
    public BaseResponse<Boolean> bindLogisticOrder(@RequestParam @ApiParam("辅材申请单id") int id,@RequestParam @ApiParam("物流单号") String logisticOrder){

        var before=this.tauxiliarymaterialsMapper.selectByPrimaryKey(id);
        if(before==null){
            throw new ServiceException(400,"id erroe");
        }
        if(StringUtils.isEmpty(before.getLogisticOrder())){
            return new BaseResponse<>(this.tauxiliarymaterialsMapper.updateLogisticOrder(id,logisticOrder)>0);
        }else{
            List<String> los=Arrays.stream(before.getLogisticOrder().split(",")).collect(Collectors.toList());
            los.add(logisticOrder);
            return new BaseResponse<>(this.tauxiliarymaterialsMapper.updateLogisticOrder(id,String.join(",",los.stream().distinct().collect(Collectors.toList())))>0);
        }
    }
    @PostMapping("/batchbindLogisticOrder")
    @ApiOperation(value = "批量绑定物流单号")
    public BaseResponse<Boolean> bindLogisticOrder(@RequestParam @ApiParam("辅材申请单id") int id,@RequestBody @ApiParam("物流单号列表") List<String> logisticOrders){

        var before=this.tauxiliarymaterialsMapper.selectByPrimaryKey(id);
        if(before==null){
            throw new ServiceException(400,"id erroe");
        }
        if(StringUtils.isEmpty(before.getLogisticOrder())){
            return new BaseResponse<>(this.tauxiliarymaterialsMapper.updateLogisticOrder(id,String.join(",",logisticOrders))>0);
        }else{
            var los=Arrays.stream(before.getLogisticOrder().split(",")).collect(Collectors.toList());
            los.addAll(logisticOrders);
            return new BaseResponse<>(this.tauxiliarymaterialsMapper.updateLogisticOrder(id,String.join(",",los.stream().distinct().collect(Collectors.toList())))>0);
        }
    }
    @GetMapping("/deleteLogisticOrder")
    @ApiOperation(value = "删除物流单号")
    public BaseResponse<Boolean> deleteLogisticOrder(@RequestParam @ApiParam("辅材申请单id") int id,@RequestParam @ApiParam("物流单号") String logisticOrder){

        var before=this.tauxiliarymaterialsMapper.selectByPrimaryKey(id);
        if(before==null){
            throw new ServiceException(400,"id erroe");
        }
        if(StringUtils.isEmpty(before.getLogisticOrder())){
            return new BaseResponse<>(true);
        }else{
            var los=Arrays.stream(before.getLogisticOrder().split(",")).collect(Collectors.toList());
            los.remove(logisticOrder);
            return new BaseResponse<>(this.tauxiliarymaterialsMapper.updateLogisticOrder(id,String.join(",",los))>0);
        }
    }


    @PostMapping("/export")
    @ApiOperation(value = "导出辅料申请列表")
    public void export(HttpServletResponse response,@RequestBody Tauxiliarymaterials tauxiliarymaterials) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<TauxiliarymaterialsExcelVo> data = TauxiliarymaterialsConvert.INSTANCE.convertList(this
                .tauxiliarymaterialsMapper.selectTAuxiliaryMaterialsList(tauxiliarymaterials));
        // 输出
        ExcelUtils.write(response, "辅材申请列表.xls", "辅材信息", TauxiliarymaterialsExcelVo.class, data);
    }
    @GetMapping("/getRecent3MonthHasChecked")
    @ApiOperation(value = "查询最近三个月已审批物料申请单")
    public BaseResponse<List<Tauxiliarymaterials>> getRecent3MonthHasChecked(@RequestParam @ApiParam("物料编码") String materialCode){
        TauxiliarymaterialsCriteria criteria=new TauxiliarymaterialsCriteria();
        criteria.createCriteria().andApplyTimeGreaterThanOrEqualTo(LocalDateTime.now().minusMonths(3))
                .andMaterialCodeEqualTo(materialCode)
                .andApplyStatusEqualTo(TauxiliarymaterialStatusEnum.退货已通过.name());
        return new BaseResponse<>(this.tauxiliarymaterialsMapper.selectByExample(criteria));
    }
    @GetMapping("/getRecentHasChecked")
    @ApiOperation(value = "查询最近一个已审批物料申请单")
    public BaseResponse<Tauxiliarymaterials> getRecentHasChecked(@RequestParam @ApiParam("物料编码") String materialCode){
        return new BaseResponse<Tauxiliarymaterials>(this.tauxiliarymaterialsMapper.getRecentTAuxiliaryMaterials(materialCode));
    }
    @GetMapping("/queryLogisticOrder")
    @ApiOperation(value = "根据申请单号查询物流单号")
    public BaseResponse<List<String>> queryLogisticOrder(Delivery delivery){
        List<String> listStatus= Arrays.asList("已发货","已收货","退货已申请","退货已驳回","退货已通过","退货完成");
        delivery.setLogisticStatusList(listStatus);
        List<Delivery> list=this.deliveryMapper.selectDeliveryList(delivery);
        List<String> listApplyOrder=new ArrayList<>();
        for (Delivery delivery1:list){
            if (!StringUtils.isEmpty(delivery1)){
                listApplyOrder.add(delivery1.getLogisticOrder());
            }
        }
        return new BaseResponse(listApplyOrder);
    }
}
